==> apache-devlake-0.21.0-incubating-src.tar.gz: apache-devlake-0.21.0-incubating-src/backend/plugins/circleci/tasks/workflow_collector.go <==
/*
Licensed to the Apache Software Foundation (ASF) under one or more
contributor license agreements.  See the NOTICE file distributed with
this work for additional information regarding copyright ownership.
The ASF licenses this file to You under the Apache License, Version 2.0
(the "License"); you may not use this file except in compliance with
the License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package tasks

import (
	"encoding/json"
	"github.com/apache/incubator-devlake/core/dal"
	"github.com/apache/incubator-devlake/core/errors"
	"github.com/apache/incubator-devlake/core/plugin"
	"github.com/apache/incubator-devlake/helpers/pluginhelper/api"
	"github.com/apache/incubator-devlake/plugins/circleci/models"
	"net/http"
	"net/url"
	"reflect"
)

const RAW_WORKFLOW_TABLE = "circleci_api_workflows"

var _ plugin.SubTaskEntryPoint = CollectWorkflows

var CollectWorkflowsMeta = plugin.SubTaskMeta{
	Name:             "collectWorkflows",
	EntryPoint:       CollectWorkflows,
	EnabledByDefault: true,
	Description:      "collect circleci workflows",
	DomainTypes:      []string{plugin.DOMAIN_TYPE_CICD},
}

func CollectWorkflows(taskCtx plugin.SubTaskContext) errors.Error {
	rawDataSubTaskArgs, data := CreateRawDataSubTaskArgs(taskCtx, RAW_WORKFLOW_TABLE)
	logger := taskCtx.GetLogger()
	logger.Info("collect workflows")

	clauses := []dal.Clause{
		dal.Select("id"),
		dal.From(&models.CircleciPipeline{}),
		dal.Where("_tool_circleci_pipelines.connection_id = ? and _tool_circleci_pipelines.project_slug = ? ", data.Options.ConnectionId, data.Options.ProjectSlug),
	}

	db := taskCtx.GetDal()
	cursor, err := db.Cursor(clauses...)
	if err != nil {
		return err
	}
	iterator, err := api.NewDalCursorIterator(db, cursor, reflect.TypeOf(models.CircleciPipeline{}))
	if err != nil {
		return err
	}

	collector, err := api.NewApiCollector(api.ApiCollectorArgs{
		RawDataSubTaskArgs: *rawDataSubTaskArgs,
		ApiClient:          data.ApiClient,
		UrlTemplate:        "/v2/pipeline/{{ .Input.Id }}/workflow",
		Input:              iterator,
		GetNextPageCustomData: func(prevReqData *api.RequestData, prevPageResponse *http.Response) (interface{}, errors.Error) {
			res := CircleciPageTokenResp[any]{}
			err := api.UnmarshalResponse(prevPageResponse, &res)
			if err != nil {
				return nil, err
			}
			if res.NextPageToken == "" {
				return nil, api.ErrFinishCollect
			}
			return res.NextPageToken, nil
		},
		Query: func(reqData *api.RequestData) (url.Values, errors.Error) {
			query := url.Values{}
			if pageToken, ok := reqData.CustomData.(string); ok && pageToken != "" {
				query.Set("page_token", reqData.CustomData.(string))
			}
			return query, nil
		},
		ResponseParser: func(res *http.Response) ([]json.RawMessage, errors.Error) {
			data := CircleciPageTokenResp[[]json.RawMessage]{}
			err := api.UnmarshalResponse(res, &data)
			return data.Items, err
		},
	})
	if err != nil {
		logger.Error(err, "collect workflows error")
		return err
	}
	return collector.Execute()
}

==> apache-devlake-1.0.1-incubating-src.tar.gz: apache-devlake-1.0.1-incubating-src/backend/plugins/circleci/tasks/workflow_collector.go <==
/*
Licensed to the Apache Software Foundation (ASF) under one or more
contributor license agreements.  See the NOTICE file distributed with
this work for additional information regarding copyright ownership.
The ASF licenses this file to You under the Apache License, Version 2.0
(the "License"); you may not use this file except in compliance with
the License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package tasks

import (
	"github.com/apache/incubator-devlake/core/dal"
	"github.com/apache/incubator-devlake/core/errors"
	"github.com/apache/incubator-devlake/core/plugin"
	"github.com/apache/incubator-devlake/helpers/pluginhelper/api"
	"github.com/apache/incubator-devlake/plugins/circleci/models"
	"reflect"
)

const RAW_WORKFLOW_TABLE = "circleci_api_workflows"

var _ plugin.SubTaskEntryPoint = CollectWorkflows

var CollectWorkflowsMeta = plugin.SubTaskMeta{
	Name:             "collectWorkflows",
	EntryPoint:       CollectWorkflows,
	EnabledByDefault: true,
	Description:      "collect circleci workflows",
	DomainTypes:      []string{plugin.DOMAIN_TYPE_CICD},
}

func CollectWorkflows(taskCtx plugin.SubTaskContext) errors.Error {
	rawDataSubTaskArgs, data := CreateRawDataSubTaskArgs(taskCtx, RAW_WORKFLOW_TABLE)
	logger := taskCtx.GetLogger()
	logger.Info("collect workflows")

	clauses := []dal.Clause{
		dal.Select("id"),
		dal.From(&models.CircleciPipeline{}),
		dal.Where("_tool_circleci_pipelines.connection_id = ? and _tool_circleci_pipelines.project_slug = ? ", data.Options.ConnectionId, data.Options.ProjectSlug),
	}

	db := taskCtx.GetDal()
	cursor, err := db.Cursor(clauses...)
	if err != nil {
		return err
	}
	iterator, err := api.NewDalCursorIterator(db, cursor, reflect.TypeOf(models.CircleciPipeline{}))
	if err != nil {
		return err
	}

	collector, err := api.NewApiCollector(api.ApiCollectorArgs{
		RawDataSubTaskArgs:    *rawDataSubTaskArgs,
		ApiClient:             data.ApiClient,
		UrlTemplate:           "/v2/pipeline/{{ .Input.Id }}/workflow",
		Input:                 iterator,
		GetNextPageCustomData: ExtractNextPageToken,
		Query:                 BuildQueryParamsWithPageToken,
		ResponseParser:        ParseCircleciPageTokenResp,
	})
	if err != nil {
		logger.Error(err, "collect workflows error")
		return err
	}
	return collector.Execute()
}
